/********************************************************************\
 * This is an example file for Mu (Miscellaneous Utilities).        *
 *                                                                  *
 * You may copy, modify, and redistribute this example under the    *
 * terms of the GNU General Public License version 3 (or, at your   *
 * option, any later version), and/or the GNU Free Documentation    *
 * License, Version 1.3 (or, at your option, any later version).    *
 *                                                                  *
 * This example is free software: you can redistribute it and/or    *
 * modify it under the terms of the GNU General Public License as   *
 * published by the Free Software Foundation, either version 3 of   *
 * the License, or (at your option) any later version.              *
 *                                                                  *
 * This example is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU General Public License for more details.                     *
 *                                                                  *
 * You should have received a copy of the GNU General Public        *
 * License along with this example.  If not, see                    *
 * <https://www.gnu.org/licenses/>.                                 *
 *                                                                  *
 * Permission is granted to copy, distribute and/or modify these    *
 * examples under the terms of the GNU Free Documentation License,  *
 * Version 1.3 or any later version published by the Free Software  *
 * Foundation; with no Invariant Sections, no Front-Cover Texts,    *
 * and no Back-Cover Texts.  A copy of the license is included in   *
 * the file ../gfdl.texi.                                           *
\********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <mu/options.h>

extern char **environ;

/* These are messages to output in the help text. */
#define USAGE "[OPTION]..."
#define DESCRIPTION                                             \
  "Do stuff. If this text is really long, it will be wrapped. " \
  "Some more text to make this text long enough to be wrapped."
#define NOTES "Report bugs to Asher Gordon <AsDaGo@posteo.net>."

int print_version(void *data, char *err) {
  puts("Version 1.0");
  exit(0);
}

int main(int argc, char **argv) {
  int ret;
  MU_OPT options[] = {
    {
     .short_opt = 'n',
     .long_opt  = "none",
     .has_arg   = MU_OPT_NONE,
     .help      = "an option which takes no argument"
    },
    {
     .short_opt = 'o',
     .long_opt  = "optional",
     .has_arg   = MU_OPT_OPTIONAL,
     .arg_type  = MU_OPT_STRING,
     .arg_help  = "OPTARG",
     .help      = "an option which optionally takes an argument"
    },
    {
     .short_opt = 'r',
     .long_opt  = "required",
     .has_arg   = MU_OPT_REQUIRED,
     .arg_type  = MU_OPT_STRING,
     .arg_help  = "REQARG",
     .help      = "an option which requires an argument"
    },
    /* So mu_create_help_option knows where to insert the help
       option. */
    { 0 },
    {
     /* Options can appear after the help option. */
     .short_opt = 'v',
     .long_opt  = "version",
     .has_arg   = MU_OPT_NONE,
     .callback_none = print_version,
     .help      = "print version information and exit"
    },
    { 0 }       /* So `mu_parse_opts' knows where to stop */
  };

  /* Create the help option. MU_HELP_BOTH indicates that we want
     both a short help option (-h) and a long one (--help). */
  mu_create_help_option(options, argv[0], USAGE,
                        DESCRIPTION, NOTES, MU_HELP_BOTH);

  /* Parse the options. */
  ret = mu_parse_opts(argc, argv, environ, options,
                      MU_OPT_BUNDLE | MU_OPT_PERMUTE);
  if (MU_OPT_ERR(ret)) {
    /* Error! If the error was an option parsing error, print a
       usage message so the user knows how to use us properly. */
    if (ret == MU_OPT_ERR_PARSE)
      mu_format_help(stderr, argv[0], USAGE,
                     DESCRIPTION, NOTES, options);
    return 1;
  }

  return 0;
}
